/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F3910_Flash
 * IDE tooling  : AndeSight C/C++ IDE, version: 3.2.1
 * Device Ver.  :
 * Library Ver. : 0.5
 * MCU Device   :
 * Description  :
 * Created Date : 2022/03/22
 * Created by   :
 *
 * Program Description:
 *
 *   HY16F3910                LCD Board(HY10000-AM01)                                                                       
 *----------------          ----------------
 *                |        |                                             
 *  PT13.4~PT13.5 | -----> | SEG0~SEG1                                                                                                             
 *    PT6.0~PT6.7 | -----> | SEG2~SEG9                                                                                                             
 *    PT7.0~PT7.7 | -----> | SEG10~SEG17                                                                                                                                                                                                           
 *  PT13.0~PT13.3 | -----> | COM0~COM3                                                                                                             
 *                |        |                                             
 *----------------         ----------------  
 * 
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY391apb.h"
#include "DrvLCD.h"
#include "DrvClock.h"
#include "DrvPMU.h"
#include "Display.h"
#include "main.h"
#include "System.h"
#include "BootFunc.h"
#include "DrvFlash.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//#define HAO_4MHZ
#define HAO_32MHZ

/*----------------------------------------------------------------------------*/
/* Data Flash                                                                 */
/*----------------------------------------------------------------------------*/
const unsigned char DefaultData[256] __attribute__ ((section ("DATA_EARE0"))) =
{
0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F,
0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F,
0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F,
0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F,
0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x7F,
0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x8B, 0x8C, 0x8D, 0x8E, 0x8F,
0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0x9B, 0x9C, 0x9D, 0x9E, 0x9F,
0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF,
0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF,
0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF,
0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF,
0xE0, 0xE1, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xEB, 0xEC, 0xED, 0xEE, 0xEF,
0xF0, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE, 0xFF
};  //0xAFC00~0xAFC
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{
  unsigned char index,error;
  unsigned int BufferTx[32];
  int BufferRx[32];
  DrvPMU_VDD15Trim();  // Make sure VDD15 Voltage,before change OSC setting
  DrvCLOCK_EnableHighOSC(E_INTERNAL,100);     // Select HSRC
  #if defined(HAO_4MHZ)
  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_4M);                 //Select internal 4.147MHz, and calibration 4.147MHz
  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV1);                //ENMCD=01b,MCU Clock/1, CPU CLOCK IS 'HS_CK/1'
  #elif defined(HAO_32MHZ)
  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_32M);                 //Select internal 31.795MHz, and calibration 31.795MHz
  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV2);                //ENMCD=00b,MCU Clock/2, CPU CLOCK IS 'HS_CK/2'
  #endif

  DisplayInit();
  ClearLCDframe();
  Delay(10000);
  DisplayHYcon();
  Delay(10000);

  SYS_DisableGIE();                      //before execute Flash burn, must be to do that Disable GIE

  //BurnPage
  //demo1
  for(index=0;index<32;index++)
  {
    BufferTx[index]=index;               //to set BufferTx[0]=0x0....BufferTx[31]=0x1f
    BufferRx[index]=0xFFFFFFFF;          //to set BufferRx[0]=0xFFFFFFFF....BufferRx[31]=0xFFFFFFFF
  }
  ISP_FUNC_ROMP->FlashOpEn();                    //Flash write enable.
  ISP_FUNC_ROMP->SectorErase(0xAFC00);           //Flash erase 1KB sector, erase 0xAFC00~0xAFFFF
  ISP_FUNC_ROMP->BurnPage(0xAFC00,BufferTx,32);  //Flash page write, write address 0xAFC00~0xAFC7F
  ReadPage(0xAFC00,BufferRx);                    //Flash page read, read address 0xAFC00~0xAFC7F
  ISP_FUNC_ROMP->FlashOpDis();                   //Flash write disable.

  //verify the ROM_BurnPage function, if fail to show "1" on the LCD Display
  for(index=0;index<32;index++)
  {
    if(BufferTx[index]!=BufferRx[index])
    {
       LCD_DATA_DISPLAY(1);
       while(1);
    }
  }

  //demo2
  for(index=0;index<32;index++)
  {
    BufferTx[index]=31-index;            //to set BufferTx[0]=0x1f....BufferTx[31]=0x0
    BufferRx[index]=0xFFFFFFFF;          //to set BufferRx[0]=0xFFFFFFFF....BufferRx[31]=0xFFFFFFFF
  }
  ISP_FUNC_ROMP->FlashOpEn();                    //Flash write enable.
  //ISP_FUNC_ROMP->SectorErase(0xAFC80);           //Flash erase 1KB sector, erase 0xAFC00~0xAFFFF
  ISP_FUNC_ROMP->BurnPage(0xAFC80,BufferTx,32);  //Flash page write, write address 0xAFC80~0xAFCFF
  ReadPage(0xAFC80,BufferRx);                    //Flash page read, read address 0xAFC80~0xAFCFF
  ISP_FUNC_ROMP->FlashOpDis();                   //Flash write disable.

  //verify the ROM_BurnPage function, if fail to show "1" on the LCD Display
  for(index=0;index<32;index++)
  {
    if(BufferTx[index]!=BufferRx[index])
    {
       LCD_DATA_DISPLAY(1);
       while(1);
    }
  }

  //BurnWord
  //demo1
  ISP_FUNC_ROMP->FlashOpEn();                         //Flash write enable.
  //ISP_FUNC_ROMP->SectorErase(0xAFC00);                //Flash erase 1KB sector
  error=ISP_FUNC_ROMP->BurnWord(0xAFD00,0x12345678);  //Flash word write, write address 0xAFD00
  if(error==0)                                        //if Error=0, it means BurnWord success
  {
    BufferRx[0]=ReadWord(0xAFD00);
    ISP_FUNC_ROMP->FlashOpDis();                      //Flash write disable.
  }
  if(error==3)                                        //if Error=3, it means BurnWord failure
  {
    LCD_DATA_DISPLAY(1);
    while(1);
  }

  //demo2
  ISP_FUNC_ROMP->FlashOpEn();                         //Flash write enable.
  //ISP_FUNC_ROMP->SectorErase(0xAFC00);                //Flash erase 1KB sector
  error=ISP_FUNC_ROMP->BurnWord(0xAFD04,0xFFFFFF50);  //Flash word write, write address 0xAFD04
  if(error==0)                                        //if Error=0, it means BurnWord success
  {
    BufferRx[1]=ReadWord(0xAFD04);
    ISP_FUNC_ROMP->FlashOpDis();                      //Flash write disable.
  }
  if(error==3)                                        //if Error=3, it means BurnWord failure
  {
    LCD_DATA_DISPLAY(1);
    while(1);
  }


  //demo3
  ISP_FUNC_ROMP->FlashOpEn();                         //Flash write enable.
  //ISP_FUNC_ROMP->SectorErase(0xAFC00);                //Flash erase 1KB sector
  error=ISP_FUNC_ROMP->BurnWord(0xAFD08,0xABCD5678);  //Flash word write, write address 0xAFD08
  if(error==0)                                        //if Error=0, it means BurnWord success
  {
    BufferRx[2]=ReadWord(0xAFD08);
    ISP_FUNC_ROMP->FlashOpDis();                      //Flash write disable.
  }
  if(error==3)                                        //if Error=3, it means BurnWord failure
  {
    LCD_DATA_DISPLAY(1);
    while(1);
  }

  while(1);

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : LVD & BOR2 interrupt Service Routine (HW3).                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW9_ISR()                                                   */
/* Description  : PT3 interrupt Service Routine (HW9).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW9_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: general_exception_handler()                                 */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void general_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Function Name: debug_exception_handler()                                   */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void debug_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}
/*----------------------------------------------------------------------------*/
/* End Of File                                                               */
/*----------------------------------------------------------------------------*/
