/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F3910_SPI
 * IDE tooling  : AndeSight C/C++ IDE, version: 3.2.1
 * Device Ver.  :
 * Library Ver. : 0.5
 * MCU Device   :
 * Description  :
 * Created Date : 2022/03/22
 * Created by   :
 *
 * Program Description:
 * --------------------
 * HY16F3910             HY3106
 * ---------            --------
 * VDD      |           |  VDD |
 * SPI_CS   |<--------> |  CS  |
 * SPI_MISO |<--------> |  DO  |
 * SPI_CLK  |<--------> |  CK  |
 * SPI_MOSI |<--------> |  DI  |
 * VSS      |           |  VSS |
 * ---------            --------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY391apb.h"
#include "DrvLCD.h"
#include "Display.h"
#include "main.h"
#include "System.h"
#include "DrvSPI32.h"
#include "DrvGPIO.h"
#include "DrvPMU.h"
#include "DrvCLOCK.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define  Flash_PORT E_PT2
#define  Flash_CS   BIT0
#define  Flash_SCLK BIT1
#define  Flash_MISO BIT2
#define  Flash_MOSI BIT3
#define  SCS_PIN 0

//OSC
//#define HSXT  //External 16MHz
#define HSRC  //Internal HAO
//#define HAO_4MHZ
#define HAO_32MHZ

//#define SPI_4WIRE
#define SPI_3WIRE

#define SPI_save_power  //If NO SPI RX Interrupt, enter Idle mode for saving power
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
//SPI
unsigned int spi_read_buffer[128];
int ADCData;
unsigned int i;
unsigned int SPI_WRITE8;
unsigned int SPI_WRITE16;
unsigned int SPI_WRITE32;
unsigned int rxIntHappened=0;
unsigned char dummyread=0;
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
//SPI
void spi_write_8bits(uint32_t spi_address, uint32_t spi_data);
void spi_write_16bits(uint32_t spi_address, uint32_t spi_data);
unsigned int spi_read_8bits(uint32_t spi_address);
unsigned int spi_read_16bits(uint32_t spi_address);
unsigned int spi_read_24bits(uint32_t spi_address);
void InitalSPI(void);
void enterIdleMode(void);  //enter Idle mode, save power
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{
  SPI_WRITE8=0;
  SPI_WRITE16=0;
  SPI_WRITE32=0;
  DrvPMU_VDD15Trim();  // Make sure VDD15 Voltage,before change OSC setting
#if defined(HSRC)
	  DrvCLOCK_EnableHighOSC(E_INTERNAL,100);     // Select HSRC
	#if defined(HAO_4MHZ)
	  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_4M);                 //Select internal 4.147MHz, and calibration 4.147MHz
	  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV1);                //ENMCD=01b,MCU Clock/1, CPU CLOCK IS 'HS_CK/1'
	#elif defined(HAO_32MHZ)
	  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_32M);                 //Select internal 31.795MHz, and calibration 31.795MHz
	  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV2);                //ENMCD=00b,MCU Clock/2, CPU CLOCK IS 'HS_CK/2'
	#endif
#endif
#if defined(HSXT)
  DrvCLOCK_SelectOHS_HS(1);   //0:HSXT<4MHZ 1:HSXT>4MHZ
  DrvCLOCK_EnableHighOSC(E_EXTERNAL,50);
#endif

  //DISPLAY
  DisplayInit();
  ClearLCDframe();
  Delay(10000);
  DisplayHYcon();
  Delay(10000);

  //SPI
  for(i=0;i<=128;i++)
  {
     spi_read_buffer[i]=0;               //Initial SPI data buffer=0
  }
  InitalSPI();
  SYS_EnableGIE(4,0x3BF);                //Enable GIE(Global Interrupt)
  DrvSPI32_EnableRxInt();

  //If control HY3106,
  //address 0x00, it is 8bits.
  //address 0x01, it is 16bits.
  //address 0x10, it is 24bits.
  spi_00=0x00000C04; //CPHA=0b, CPOL=1b, write
  spi_write_8bits(0x00,0xF0);  //0xF0:INOSC=1b, LDO=11b, ENLDO=1b, REFOS=0b, SDOH=0b, CH=0b, TS=0b

  spi_00=0x00000C0C; //CPHA=1b, CPOL=1b, read
  spi_read_buffer[0]=spi_read_8bits(0x80);  //If correct, it is 0xF0.

  spi_00=0x00000C04; //CPHA=0b, CPOL=1b
  spi_write_16bits(0x10,0x0035);  //0x0035:DCSET=0000b, INX=00b, ADGN=00b=x1, PGA=001b=x8, FRb=1b=x1/2, OSR=01b=80sps, ENRB=0b, ADCEN=1b

  spi_00=0x00000C0C; //CPHA=1b, CPOL=1b,
  spi_read_buffer[1]=spi_read_16bits(0x90); //If correct, it is 0x35

  while(1)
  {
	#if defined(HAO_4MHZ)
    Delay(80000);  //Waiting for HY3106 ADC convert time
    #endif
    #if defined(HAO_32MHZ)
    Delay(160000);  //Waiting for HY3106 ADC convert time
    #endif
    //HY3106 : ADGN=x1, PGA=x8, FRb=x1/2, VDDA=2.4V.
    //ADC Vin range = +/- 0.15V
    spi_read_buffer[2]=spi_read_24bits(0xA0);
    ADCData=spi_read_buffer[2];
    LCD_DATA_DISPLAY(ADCData);
  }

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  //SPI RX interrupt
  while(DrvSPI32_GetRxIntFlag())
  {
    dummyread =  (unsigned char) DrvSPI32_Read();
    while((spi_00>>BUF) & 0x01);  // Wait SPI Busy
    DrvSPI32_ClrIntRxFlag();
    rxIntHappened=1;
  }
}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : LVD & BOR2 interrupt Service Routine (HW3).                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW9_ISR()                                                   */
/* Description  : PT3 interrupt Service Routine (HW9).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW9_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: general_exception_handler()                                 */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void general_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Function Name: debug_exception_handler()                                   */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void debug_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}
/*---------------------------------------------------------------------------*/
/* SPI                                                                       */
/*---------------------------------------------------------------------------*/
static void doClearSPIFlag(void)
{
  rxIntHappened=0;
}
static void waitSPIDONE(void)
{
  while(!rxIntHappened)
  {
    #if defined(HAO_4MHZ)
    enterIdleMode();
    #else
    asm("NOP");
    #endif
  }
}

#if defined(SPI_4WIRE)
void spi_write_8bits(uint32_t spi_address, uint32_t spi_data)
{
  DrvSPI32_BitLength(16);    //write MSB 16bits
  SPI_WRITE8=(spi_address<<8) + spi_data;
  doClearSPIFlag();
  DrvSPI32_Write(SPI_WRITE8);
  waitSPIDONE();
}
void spi_write_16bits(uint32_t spi_address, uint32_t spi_data)
{
  DrvSPI32_BitLength(24);   //write MSB 24bits
  SPI_WRITE16=(spi_address<<16) + spi_data;
  doClearSPIFlag();
  DrvSPI32_Write(SPI_WRITE16);
  waitSPIDONE();
}
unsigned int spi_read_8bits(uint32_t spi_address)
{
  unsigned int au32DestinationData=0;
  DrvSPI32_BitLength(16);   //write MSB 16bits
  doClearSPIFlag();
  DrvSPI32_Write((spi_address<<8) + 0xFF);
  waitSPIDONE();
  au32DestinationData=DrvSPI32_Read();
  return(au32DestinationData&0x000000FF);
}
unsigned int spi_read_16bits(uint32_t spi_address)
{
  unsigned int au32DestinationData=0;
  DrvSPI32_BitLength(24); //write MSB 24bits
  doClearSPIFlag();
  DrvSPI32_Write((spi_address<<16) + 0xFF);
  waitSPIDONE();
  au32DestinationData=DrvSPI32_Read();
  return(au32DestinationData&0x0000FFFF);
}
unsigned int spi_read_24bits(uint32_t spi_address)
{
  unsigned int au32DestinationData=0;
  DrvSPI32_BitLength(32);
  SPI_WRITE32=spi_address<<24;
  doClearSPIFlag();
  DrvSPI32_Write(SPI_WRITE32);
  waitSPIDONE();
  au32DestinationData=DrvSPI32_Read();
  return(au32DestinationData & 0x00FFFFFF );
}
#endif

#if defined(SPI_3WIRE)
void spi_write_8bits(uint32_t spi_address, uint32_t spi_data)
{
  DrvGPIO_ClrBit(E_PT2,SCS_PIN);              //CS: Low
  DrvSPI32_BitLength(16);
  SPI_WRITE8=(spi_address<<8) + spi_data;
  doClearSPIFlag();
  DrvSPI32_Write(SPI_WRITE8);
  waitSPIDONE();
  DrvGPIO_SetBit(E_PT2,SCS_PIN);              //CS: High
}
void spi_write_16bits(uint32_t spi_address, uint32_t spi_data)
{
  DrvGPIO_ClrBit(E_PT2,SCS_PIN);              //CS: Low
  DrvSPI32_BitLength(24);                     //write MSB 24bits
  SPI_WRITE16=(spi_address<<16) + spi_data;
  doClearSPIFlag();
  DrvSPI32_Write(SPI_WRITE16);
  waitSPIDONE();
  DrvGPIO_SetBit(E_PT2,SCS_PIN);              //CS: High
}
unsigned int spi_read_8bits(uint32_t spi_address)
{
  DrvGPIO_ClrBit(E_PT2,SCS_PIN);              //CS: Low
  unsigned int au32DestinationData=0;
  DrvSPI32_BitLength(16);                     //write MSB 16bits
  doClearSPIFlag();
  DrvSPI32_Write((spi_address<<8) + 0xFF);
  waitSPIDONE();
  au32DestinationData=DrvSPI32_Read();
  DrvGPIO_SetBit(E_PT2,SCS_PIN);              //CS: High
  return(au32DestinationData&0x000000FF);
}
unsigned int spi_read_16bits(uint32_t spi_address)
{
  DrvGPIO_ClrBit(E_PT2,SCS_PIN);              //CS: Low
  unsigned int au32DestinationData=0;
  DrvSPI32_BitLength(24);                     //write MSB 24bits
  doClearSPIFlag();
  DrvSPI32_Write((spi_address<<16) + 0xFF);
  waitSPIDONE();
  au32DestinationData=DrvSPI32_Read();
  DrvGPIO_SetBit(E_PT2,SCS_PIN);              //CS: High
  return(au32DestinationData&0x0000FFFF);
}

unsigned int spi_read_24bits(uint32_t spi_address)
{
  DrvGPIO_ClrBit(E_PT2,SCS_PIN);              //CS: Low
  unsigned int au32DestinationData=0;
  DrvSPI32_BitLength(32);
  SPI_WRITE32=spi_address<<24;
  doClearSPIFlag();
  DrvSPI32_Write(SPI_WRITE32);
  waitSPIDONE();
  au32DestinationData=DrvSPI32_Read();
  DrvGPIO_SetBit(E_PT2,SCS_PIN);              //CS: High
  return(au32DestinationData & 0x00FFFFFF );
}
#endif
/*---------------------------------------------------------------------------*/
/* POWER                                                                     */
/*---------------------------------------------------------------------------*/
void enterIdleMode(void)
{
  SOCSTATUSL = 0x1010; // IDLE
  asm("STANDBY wake_grant");
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalSPI(void)                                             */
/* Description  :                                                             */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalSPI(void)
{
  Initial_SPI32_Register();    //Reset SPI Register
  DrvSPI32_CLKSource(1);
  DrvGPIO_Open(E_PT2,Flash_MOSI|Flash_SCLK,E_IO_OUTPUT);  //MOSI=PT23, SCK=PT21
  DrvGPIO_Open(E_PT2,Flash_MISO,E_IO_INPUT);  //MISO=PT22
  DrvGPIO_Open(E_PT2,Flash_CS,E_IO_OUTPUT);  //CS=PT2.0 output
  DrvGPIO_SetPortBits(E_PT2,Flash_CS); //CS=PT2.0 high
#if defined(SPI_3WIRE)
  DrvSPI32_Open(E_DRVSPI_MASTER2,E_DRVSPI_TYPE1,2,5);
#endif
#if defined(SPI_4WIRE)
  DrvSPI32_Open(E_DRVSPI_MASTER1,E_DRVSPI_TYPE1,2,5);
#endif
  // Master2 : 3-wire mode
  // Master1 : 4-wire mode
  // Type1 : CPHA=0 CPOL=1
  // Type3 : CPHA=1 CPOL=1
  // 2 : PT2.0=CS PT2.1=CK P2.2=MISO PT2.3=MOSI
  // 5 : CLOCK/128
  DrvSPI32_Enable();
}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
