/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F3981_TimerB2
 * IDE tooling  :
 * Device Ver.  :
 * Library Ver. : 0.4
 * MCU Device   :
 * Description  : User choose #define TMATEST or TMBTEST or TMCTEST.
 * This program shows Timer count ++ on LCD Display
 * Created Date : 2022/03/22
 * Created by   : Robert.Wang
 *
 * Program Description:
 *
 *   HY16F3910                LCD Board(HY10000-AM01)                                                                       
 *----------------          ----------------
 *                |        |                                             
 *  PT13.4~PT13.5 | -----> | SEG0~SEG1                                                                                                             
 *    PT6.0~PT6.7 | -----> | SEG2~SEG9                                                                                                             
 *    PT7.0~PT7.7 | -----> | SEG10~SEG17                                                                                                                                                                                                           
 *  PT13.0~PT13.3 | -----> | COM0~COM3                                                                                                             
 *                |        |                                             
 *----------------         ----------------  
 * 
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY391apb.h"
#include "DrvCLOCK.h"
#include "DrvLCD.h"
#include "DrvTimer.h"
#include "DrvPMU.h"
#include "Display.h"
#include "main.h"
#include "System.h"

/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMB2done:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_WDTdone:1;
    unsigned REV:1;
  };
} MCUSTATUS;


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//OSC
//#define HSXT  //External 16MHz
#define HSRC  //Internal HAO
//#define HAO_4MHZ
#define HAO_32MHZ

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
MCUSTATUS  MCUSTATUSbits;
unsigned int TMB2_Count=0;

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);


/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{
  DrvPMU_VDD15Trim();  // Make sure VDD15 Voltage,before change OSC setting
#if defined(HSRC)
	  DrvCLOCK_EnableHighOSC(E_INTERNAL,100);     // Select HSRC
	#if defined(HAO_4MHZ)
	  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_4M);                 //Select internal 4.147MHz, and calibration 4.147MHz
	  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV1);                //ENMCD=01b,MCU Clock/1, CPU CLOCK IS 'HS_CK/1'
	#elif defined(HAO_32MHZ)
	  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_32M);                 //Select internal 31.795MHz, and calibration 31.795MHz
	  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV2);                //ENMCD=00b,MCU Clock/2, CPU CLOCK IS 'HS_CK/2'
	#endif
#endif
#if defined(HSXT)
  DrvCLOCK_SelectOHS_HS(1);   //0:HSXT<4MHZ 1:HSXT>4MHZ
  DrvCLOCK_EnableHighOSC(E_EXTERNAL,50);
#endif

  DisplayInit();
  DisplayHYcon();
  ClearLCDframe();
  DrvTMB2_Clk_Source(1,3);                       //Timer B Prescaler 1
                                                 //1: HS_CK,clock source.
                                                 //3: clock divider.8
  DrvTMB2_Open(E_TMB_MODE0,E_TMB_NORMAL,0xFFFF); //Timer B overflow 0xffff
  DrvTIMER_ClearIntFlag(E_TMB2);                 //Clear Timer B interrupt flag
  DrvTIMER_EnableInt(E_TMB2);                    //Timer B interrupt enable

  MCUSTATUSbits._byte = 0;
  TMB2_Count=0;
  SYS_EnableGIE(4,0x3BF);                        //Enable GIE(Global Interrupt)

  while(1)                                       //Wait for Interrupt
  {
    if(MCUSTATUSbits.b_TMB2done==1)
    {
      LCD_DATA_DISPLAY(TMB2_Count);
      MCUSTATUSbits.b_TMB2done=0;
    }
  }

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : LVD & BOR2 interrupt Service Routine (HW3).                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

  if(DrvTIMER_GetIntFlag(E_TMB2))
  {
    DrvTIMER_ClearIntFlag(E_TMB2);         // Clear TMB2 interrupt flag
    MCUSTATUSbits.b_TMB2done=1;
    TMB2_Count++;
  }

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW9_ISR()                                                   */
/* Description  : PT3 interrupt Service Routine (HW9).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW9_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: general_exception_handler()                                 */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void general_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Function Name: debug_exception_handler()                                   */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void debug_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
