/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F3910_UART2
 * IDE tooling  : AndeSight C/C++ IDE, version: 3.2.1
 * Device Ver.  :
 * Library Ver. : 0.5
 * MCU Device   :
 * Description  :
 * Created Date : 2022/03/22
 * Created by   :
 *
 * Program Description  : using #define HAO_4MHZ/HAO_32MHZ to select HAO frequency.
 * UART TX continue transmit ABCEDF until RX receiving abcedf to stop transmit. If RX receiving not equal to abcedf, UART TX start to transmit ABCEDF
 *
 *  HY16F3910                host                                                                      
 *--------------          -----------
 *              |        |                                             
 *          VDD | -----> | VDD                                         
 *           TX | -----> | RX                                                                                                             
 *           RX | <----- | TX                                                                                                             
 *          VSS | -----> | VSS      
 *              |        |                                             
 *--------------         ------------  
 * 
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY391apb.h"
#include "DrvCLOCK.h"
#include "DrvLCD.h"
#include "Display.h"
#include "main.h"
#include "System.h"
#include "DrvGPIO.h"
#include "DrvUART.h"
#include "DrvPMU.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART2_TxDone:1;
    unsigned b_UART2_RxDone:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//PORT
//#define PORT9

//OSC
//#define HSXT  //External 16MHz
#define HSRC  //Internal HAO
//#define HAO_4MHZ
#define HAO_32MHZ

#if defined(PORT9)
#define  UART2_PORT E_PT9
#define  UART2_TXD  BIT2
#define  UART2_RXD  BIT3
#else
#define  UART2_PORT E_PT2
#define  UART2_TXD  BIT2
#define  UART2_RXD  BIT3
#endif

#define  Uart2_RX_BufferSize 6
#define  Uart2_TX_BufferSize 8

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char Uart2RxBuffer[Uart2_RX_BufferSize]={0},Uart2TxBuffer[Uart2_TX_BufferSize]={0};
unsigned char Uart2TxIndex,Uart2TxLength,Uart2RxIndex,Uart2RxLength;
MCUSTATUS  MCUSTATUSbits;

unsigned char Uart2RxBuffer_Command[Uart2_RX_BufferSize]=
{
0x61,0x62,0x63,0x64,0x65,0x66 //ASCII  KEYWORD = abcdef
};
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void InitalUart2(void);
void InitalClock(void);
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{
  unsigned char Stop_To_Send_UART2=DISABLE;
  DrvPMU_VDD15Trim();  // Make sure VDD15 Voltage,before change OSC setting
  InitalClock();
  InitalUart2();
  MCUSTATUSbits._byte = 0;
  MCUSTATUSbits.b_UART2_TxDone=ENABLE;
  SYS_EnableGIE(4,0x3BF);  //Enable GIE(Global Interrupt)

  Uart2TxIndex=0;
  Uart2RxIndex=0;

  while(1)
  { Delay(50000);
    //UART2 RX
    if(MCUSTATUSbits.b_UART2_RxDone==ENABLE)
    {

      if(
        (Uart2RxBuffer[5]==Uart2RxBuffer_Command[5] && Uart2RxBuffer[4]==Uart2RxBuffer_Command[4]) &&
        (Uart2RxBuffer[3]==Uart2RxBuffer_Command[3] && Uart2RxBuffer[2]==Uart2RxBuffer_Command[2]) &&
        (Uart2RxBuffer[1]==Uart2RxBuffer_Command[1] && Uart2RxBuffer[0]==Uart2RxBuffer_Command[0])
        )
    {
        //if UART2 receive == 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f)
        //send out 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f) and stop to send out
        Stop_To_Send_UART2=ENABLE;  //UART stop to send out ABCDEF
        for(Uart2TxLength=0;Uart2TxLength<=Uart2_TX_BufferSize;Uart2TxLength++)
        {
          Uart2TxBuffer[Uart2TxLength]=Uart2RxBuffer[Uart2TxLength];
          if(Uart2TxLength==Uart2_RX_BufferSize)
          {
            Uart2RxIndex=0;
            MCUSTATUSbits.b_UART2_TxDone=DISABLE;
            DrvUART2_EnableInt(ENABLE,DISABLE);   //Enable UART2 Tx Interrupt, Disable UART2 Rx Interrup
            while(!MCUSTATUSbits.b_UART2_TxDone); //If  MCUSTATUSbits.b_UART2_TxDone=DISABLE, stop at here
          }
        }
    }
    else
    {
      //if UART2 receive != 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f)
      //User defined at here
      Stop_To_Send_UART2=DISABLE;   //UART2 start to send out ABCDEF
    }
      Uart2RxIndex=0; //When finished the data reception. Set Uart2RxIndex=0
      MCUSTATUSbits.b_UART2_RxDone=DISABLE;
    }

    //UART2 TX
    if(MCUSTATUSbits.b_UART2_TxDone==ENABLE && Stop_To_Send_UART2==DISABLE )
    {
      Uart2TxBuffer[7]='\r';
      Uart2TxBuffer[6]='\n';
      Uart2TxBuffer[5]=0x46;  //F
      Uart2TxBuffer[4]=0x45;  //E
      Uart2TxBuffer[3]=0x44;  //D
      Uart2TxBuffer[2]=0x43;  //C
      Uart2TxBuffer[1]=0x42;  //B
      Uart2TxBuffer[0]=0x41;  //A
      MCUSTATUSbits.b_UART2_TxDone=DISABLE;
      Uart2TxLength=8;
      Uart2TxIndex=0;
      DrvUART2_EnableInt(ENABLE,ENABLE);     //Enable UART2 Tx Interrupt;Enable UART2 Rx Interrupt
      while(!MCUSTATUSbits.b_UART2_TxDone);  //If  MCUSTATUSbits.b_UART2_TxDone=DISABLE, stop at here
    }

  }

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : LVD & BOR2 interrupt Service Routine (HW3).                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{
  if(DrvUART2_GetRxFlag())
  {
    Uart2RxBuffer[Uart2RxIndex]=DrvUART2_Read();
    Uart2RxIndex++;
    if(Uart2RxIndex>=Uart2_RX_BufferSize)
    {
      Uart2RxIndex=0;
      MCUSTATUSbits.b_UART2_RxDone=ENABLE;
    }
    DrvUART2_ClrRxFlag();
  }

  if(DrvUART2_GetTxFlag())
  {
    if(MCUSTATUSbits.b_UART2_TxDone==DISABLE)
    {
      DrvUART2_Write(Uart2TxBuffer[Uart2TxIndex++]);
      DrvUART2_ClrTxFlag();
      if(Uart2TxIndex>=Uart2TxLength)
      {
        DrvUART2_EnableInt(ENABLE,ENABLE);  //ENABLE UART2 Tx Interrupt, ENABLE UART2 Rx Interrupt
        MCUSTATUSbits.b_UART2_TxDone=ENABLE;
        Uart2TxIndex=0;
      }
    }
    if(MCUSTATUSbits.b_UART2_TxDone==ENABLE)
    {
      while(DrvUART2_TRStatus(3));  //wait TX finish!!
      DrvUART2_EnableInt(DISABLE,ENABLE);  //DISABLE UART2 Tx Interrupt, ENABLE UART2 Rx Interrupt
      DrvUART2_ClrTxFlag();
    }
  }
}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW9_ISR()                                                   */
/* Description  : PT3 interrupt Service Routine (HW9).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW9_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: general_exception_handler()                                 */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void general_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Function Name: debug_exception_handler()                                   */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void debug_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalClock()                                               */
/* Description  : CLOCK Initial Subroutines.                                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalClock(void)
{
  DrvPMU_VDD15Trim();  // Make sure VDD15 Voltage,before change OSC setting
#if defined(HSRC)
	  DrvCLOCK_EnableHighOSC(E_INTERNAL,100);     // Select HSRC
	#if defined(HAO_4MHZ)
	  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_4M);                 //Select internal 4.147MHz, and calibration 4.147MHz
	  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV1);                //ENMCD=01b,MCU Clock/1, CPU CLOCK IS 'HS_CK/1'
	#elif defined(HAO_32MHZ)
	  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_32M);                 //Select internal 31.795MHz, and calibration 31.795MHz
	  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV2);                //ENMCD=00b,MCU Clock/2, CPU CLOCK IS 'HS_CK/2'
	#endif
#endif
#if defined(HSXT)
  DrvCLOCK_SelectOHS_HS(1);   //0:HSXT<4MHZ 1:HSXT>4MHZ
  DrvCLOCK_EnableHighOSC(E_EXTERNAL,50);
#endif
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalUart2()                                               */
/* Description  : UART2 Initial Subroutines.                                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalUart2(void)
{

#if defined(HSRC)

  #if defined(PORT9)
    #if defined(HAO_4MHZ)
    DrvUART2_ClkEnable(1,0);//TU2CK=1b(HSRC), ENU2D=1b, UACD=000b
	DrvUART2_Open(4147,B115200,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,6);
    #endif
    #if defined(HAO_32MHZ)
    DrvUART2_ClkEnable(1,0);//UACD=011b, EUART/8, 31.795M/8=3.974375MHz
    DrvUART2_Open(31795,B115200,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,6);
  #endif
  #else //PORT2
    #if defined(HAO_4MHZ)
    DrvUART2_ClkEnable(1,0);//TU2CK=1b(HSRC), ENU2D=1b, UACD=000b
    DrvUART2_Open(4147,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
    #if defined(HAO_32MHZ)
    DrvUART2_ClkEnable(1,0);//UACD=011b, EUART/8, 31.795M/8=3.974375MHz
    DrvUART2_Open(31795,B115200,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
  #endif
#endif

  //4147 : oscillator frequency 4MHz Unit After Calibration HAO = 4147kHz
  //31795 : oscillator frequency 16MHz Unit After Calibration HAO = 31795kHz
  //None parity
  //8 data bits.
  //6 : Port 9.2 =TX, Port 9.3 =RX
  //2 : Port 2.2 =TX, Port 2.3 =RX

#if defined(HSXT)
  #if defined(PORT9)
  DrvUART2_ClkEnable(0,0);       //TU2CK=0b(HSXT), ENU2D=1b, UACD=000b, select External 16MHz
  DrvUART2_Open(16000,B115200,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,6);
  #else //PORT2
  DrvUART2_ClkEnable(0,0);      //TU2CK=0b(HSXT), ENU2D=1b, UACD=000b, select External 16MHz
  DrvUART2_Open(16000,B115200,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
  #endif
#endif

#if defined(PORT9)
  DrvGPIO_LCDIOOpen(UART2_PORT, UART2_TXD, E_IO_OUTPUT);
  DrvGPIO_LCDIOOpen(UART2_PORT, UART2_RXD, E_IO_INPUT);
#else
  DrvGPIO_Open(UART2_PORT,UART2_TXD,E_IO_OUTPUT);
  DrvGPIO_Open(UART2_PORT,UART2_RXD,E_IO_INPUT);
  DrvGPIO_Open(UART2_PORT,UART2_RXD|UART2_TXD,E_IO_PullHigh);
#endif

  DrvUART2_EnableInt(ENABLE,ENABLE); //Enable UART2 Tx Interrupt, Enable UART2 Rx Interrupt
  DrvUART2_Disable_AutoBaudrate();
  DrvUART2_Close();
  DrvUART2_Enable();
}


/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
